import os
from flask import Flask, request, Response
from config import Config
from flask_security import Security, SQLAlchemyUserDatastore, hash_password
from models_shared import db, User, Role, SweElectionResult, SweDemographicResult
from flask_wtf.csrf import CSRFProtect
from flask_talisman import Talisman
from parse_data import parse_swe_election_data, parse_swe_demographic_data


"""
Note to reader:

Some comments in /backend have been generated by ChatGPT, all comments have
been proofread by the developers and corrected whenever necessary.
"""

# Create Flask application
app = Flask(__name__)

# Load configuration settings
app.config.from_object(Config)

# Set paths used by the frontend when building the project 
app.template_folder = '../frontend/build'
app.static_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'frontend', 'build', 'static'))
app.build_folder = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'frontend', 'build'))

# Security check, ensure that the project is not running in production mode.
# Further security configurations are required for a secure deployment
if not app.config['DEBUG']:
    exit("App is not secure, security configurations for production environment must be made")


# Register the blueprints with the app
from routes_default import default as default_blueprint
app.register_blueprint(default_blueprint)

from routes_auth import auth as auth_blueprint
app.register_blueprint(auth_blueprint)


"""
# Define CORS configurations. Need some further configuration before deployment.
# This should only be enabled if the frontend is hosted from another location than the backend.
from flask_cors import CORS
CORS(app, support_credentials=True)
@app.before_request
def handle_preflight():
    if request.method == "OPTIONS":
        res = Response()
        res.headers['X-Content-Type-Options'] = 'http://localhost:3000'
        return res

# Enable CSRF on all api endpoints.
# csrf = CSRFProtect(app)
"""

# Define Content Security Policy using Flask-Talisman
talisman = Talisman(app, content_security_policy=Config.csp)

# Initialize Flask-Security with built in SQLAlchemy datastore for user and role management
# Roles are not used but required by the extension when initializing the datastore 
user_datastore = SQLAlchemyUserDatastore(db, User, Role)
app.security = Security(app, user_datastore)

# When directly running the file:
if __name__ == '__main__':
    with app.app_context():
        # Bind the database to the app
        db.init_app(app)

        # Create database tables
        db.create_all()
        
        """
        # Used for developing purposes
        if not app.security.datastore.find_user(email="test@me.com"):
            app.security.datastore.create_user(
                email="admin@admin.admin",
                username="admin",
                active=False,
                password=hash_password("admin123")
            )
        """
        
        # Parse and add Swedish Election Statistics to the database
        result = parse_swe_election_data()
        county_codes = result.keys() # Integers
        for county_code in county_codes:
            county_name = result[county_code]["county_name"]
            votes_each_year = result[county_code]["parties"]
            total_votes_all_years =  result[county_code]["total_votes"]

            db_entry = SweElectionResult(
                db_county_code = county_code,
                db_county_name = county_name,
                db_votes_each_year = votes_each_year,
                db_total_votes_all_years = total_votes_all_years
            )
            db.session.add(db_entry)
        
        # Parse and add Swedish Demographic Statistics to the database
        result = parse_swe_demographic_data()
        county_codes = result.keys() # Integers
        for county_code in county_codes:
            data = result[county_code]

            db_entry = SweDemographicResult(
                db_county_code = county_code,
                db_data = data
            )
            db.session.add(db_entry)

        # Commit all changes to the database
        db.session.commit()   

        # Start the Flask application
        app.run()