from flask import Blueprint, current_app, jsonify, request
from flask_security import auth_required, logout_user, current_user, login_user
from models_shared import SweElectionResult, SweDemographicResult, Favorites, db, User
import json

auth = Blueprint("auth", __name__)

@auth.route("/api/logout", methods=["POST"])
@auth_required()
def logout():
    """
    Log out the current user.

    This route requires the user to be authenticated. It logs out the user
    and returns a JSON response indicating success.
    """
    
    logout_user()
    return jsonify(success=True)

@auth.route("/api/authenticate", methods=["POST"])
@auth_required()
def authenticate():
    """
    Logs in the current user.

    This route requires the user to be authenticated. It logs in the user
    and returns a JSON response indicating success.
    """

    query_user = User.query.filter_by(fs_uniquifier=current_user.fs_uniquifier).first()
    login_user(query_user, remember=False)

    return jsonify(success=True)


@auth.route("/api/swedish_election_party_votes", methods=["GET"])
@auth_required()
def get_swe_election_data():
    """
    Retrieve Swedish election data for a specific county.

    This route expects a 'code' query parameter representing the county code.
    It queries the database for election data corresponding to that county code
    and returns the results in JSON format.
    """

    county_code = request.args.get('code')

    if not county_code:
        return jsonify({"errors" : ["County code is required"], "success" : False})

    try:
        query_results = SweElectionResult.query.filter_by(db_county_code=int(county_code)).first()
        response = {
            "county_code": query_results.db_county_code,
            "county_name": query_results.db_county_name,
            "query_data": query_results.db_votes_each_year
        }

        return jsonify({"info": response, "success": True})

    except:
        return jsonify({"errors" : ["County code is invalid"], "success" : False})


@auth.route("/api/swedish_election_total_results", methods=["GET"])
@auth_required()
def get_swe_election_total_results():
    """
    Retrieve total Swedish election results for a specific county.

    This route expects a 'code' query parameter representing the county code.
    It queries the database for the total election results corresponding to that
    county code and returns the results in JSON format.
    """

    county_code = request.args.get('code')

    if not county_code:
        return jsonify({"errors" : ["County code is required"], "success" : False})

    try:
        query_results = SweElectionResult.query.filter_by(db_county_code=int(county_code)).first()
        
        response = {
            "county_code": query_results.db_county_code,
            "query_data": query_results.db_total_votes_all_years
        }
        return jsonify({"info": response, "success": True})
    
    except:
        return jsonify({"errors" : ["County code is invalid"], "success" : False})


@auth.route("/api/swedish_demographic", methods=["GET"])
@auth_required()
def get_swe_demographic_data():
    """
    Retrieve demographic data for a specific county in Sweden.

    This route expects a 'code' query parameter representing the county code.
    It queries the database for demographic data corresponding to that county
    code and returns the results in JSON format.
    """

    county_code = request.args.get('code')

    if not county_code:
        return jsonify({"errors" : ["County code is required"], "success" : False})

    try: 
        query_results = SweDemographicResult.query.filter_by(db_county_code=int(county_code)).first()
        response = {
            "county_code": query_results.db_county_code,
            "query_data": query_results.db_data
        }
        return jsonify({"info": response, "success": True})
    
    except:
        return jsonify({"errors" : ["County code is invalid"], "success" : False})
    

@auth.route("/api/save_favorite", methods=["POST"])
@auth_required()
def save_favorite():
    """
    Save a favorite option for the current user.

    This route expects a JSON payload from the client containing the favorite options.
    If the user has no existing favorites, a new entry is created. Otherwise, the
    new favorite is added to the existing list. The route checks for duplicate
    names and returns an error if a duplicate is found.
    """

    data = request.get_json() # Get arguments from front-end (selectedOptions)

    # Check if there is an entry for this user
    query_fav = Favorites.query.filter_by(user_fs_uniquifier=current_user.fs_uniquifier).first()

    if query_fav is None:
        # Create an entry if this is the first time the user saves a favorite
        db_entry = Favorites(
            user_fs_uniquifier = current_user.fs_uniquifier,
            favorite_list = json.dumps({'data' : [data]}) # parse the data to a json string
        )
        db.session.add(db_entry) # Add the entry to be saved
        db.session.commit() # Save
    else:
        current_dict = json.loads(query_fav.favorite_list) # Retrieve the entry and parse the json string

        existing_favorite_names = []
        for favorite in current_dict['data']: # current_dict['data'] is a list of dicts
            if 'favoriteName' in favorite.keys():
                existing_favorite_names.append(favorite['favoriteName'])

        print("existing_favorite_names")
        print(existing_favorite_names)

        if 'favoriteName' in data.keys(): 
            if data['favoriteName'] in existing_favorite_names:
                print("Name exists")
                return jsonify({"errors" : ["Name already exists"], "success" : False})

        current_dict['data'].append(data) # Add the new favorite to the list
        query_fav.favorite_list = json.dumps(current_dict) # Replace the previous list with the updated
        db.session.commit() # Save
        
    return jsonify({"success": True})


@auth.route("/api/get_favorites", methods=["GET"])
@auth_required()
def get_favorites():
    """
    Retrieve the list of favorite options for the current user.

    This route queries the database for the user's saved favorites and returns them
    in JSON format. If the user has no saved favorites, an empty list is returned.
    """
    # Check if there is an entry for this user
    query_fav = Favorites.query.filter_by(user_fs_uniquifier=current_user.fs_uniquifier).first()
    if query_fav is None:
        # This user has no favorites saved
        return jsonify({"info": {}, "success": True})
    else:
        parsed_info = json.loads(query_fav.favorite_list) # Retrieve the entry and parse the json string 
        # Only return the list is the entry
        return jsonify({"info": parsed_info['data'], "success": True})


