import re
from flask import Blueprint
from flask import render_template, jsonify
from flask import current_app, request 
from flask_security import hash_password, login_user, verify_password

import email_validator
email_validator.SPECIAL_USE_DOMAIN_NAMES.remove("test")

default = Blueprint("default", __name__)

@default.route("/", methods=["GET"])
def index():
    """
    Renders the index.html template for the root URL.
    """

    if request.method == 'GET':
        return render_template('index.html')


@default.route("/dashboard", methods=["GET"])
def index_reload():
    """
    Renders the index.html template for the root URL.

    Allow reloads to get to the application
    """

    if request.method == 'GET':
        return render_template('index.html')
    

@default.route("/api/register", methods=["POST", "OPTIONS"])
def register():
    """
    Handles user registration by validating and creating a new user.

    Validates email, username, and password before registering a user.
    If validation passes and the email/username are unique, a new user is created and logged in.
    If any validation fails, appropriate error messages are returned in the response.
    """

    if request.method == 'POST':
        data = request.get_json() 
        
        _validated_email = None
        _validated_username = None
        _validated_password = None

        errors = []

        # Validate email
        try:
             _validated_email = current_app.security._mail_util.validate(data["email"])
        except ValueError as e:
            _validated_email = None    
        if _validated_email is None: # Unsuccessfully parsed Email 
            errors.append("Email is not valid.")    
        
        # Validate username
        regex__name_pattern = "^[a-zA-Z0-9]{3,20}" 
        match_name = re.fullmatch(regex__name_pattern, data["username"])
        if match_name: 
            ubad, unorm = current_app.security._username_util.validate(data["username"])
            if ubad is None: 
                _validated_username = unorm
            else:
                errors.append(ubad)
        
        # Validate password
        regex_passw_pattern = "^(?=.*[a-zA-Z\w\W])(?=.*[0-9])[A-Za-z0-9\w\W]{4,255}$" 
        match_passwd = re.fullmatch(regex_passw_pattern, data["password"]) 
        if match_passwd:
            pbad, pnorm = current_app.security._password_util.validate(data["password"], False)
            if pbad is None:
                _validated_password = pnorm
            else:
                errors.append(pbad)
        else:
             errors.append("Password invalid. Password must contain at least one number and be between 5 and 255 characters.")
        
                
        if _validated_email and _validated_username and _validated_password:
            # Check for existing user with that email or name
            check_existing_email = current_app.security.datastore.find_user(email=_validated_email)
            check_existing_username = current_app.security.datastore.find_user(username=_validated_username)    
            if check_existing_email is None and check_existing_username is None:
                # Create a new user
                user = current_app.security.datastore.create_user(
                        active = True,
                        authenticated = True,
                        email = _validated_email,
                        username = _validated_username,
                        password = hash_password(_validated_password))
                current_app.security.datastore.commit()

                # This will login the user and store a remember me cookie on the clients machine.
                login_user(user, remember=False)
                
                # user = current_app.security.datastore.find_user(username=_validated_username)   
                # User successfully registered, confidential data should be sent here or when the user
                # performs API calls to protected endpoints.
                return jsonify(success=True)
                

            else: 
                # User already exists
                user_exist_errors = []
                if check_existing_email is not None:
                    email_exist_error = "This email address is already registered. Please use a different email."
                    user_exist_errors.append(email_exist_error)
                if check_existing_username is not None:
                    username_exist_error = "This username is already taken. Please choose a different username."
                    user_exist_errors.append(username_exist_error)
                return jsonify({"errors" : user_exist_errors, "success" : False})
       
        else: 
            return jsonify({"errors" : errors, "success" : False})



@default.route("/api/login", methods=["POST"])
def login():
    """
    Handles user login by verifying username and password.
    
    If credentials are valid, the user is logged in. If invalid, 
    appropriate error messages are returned.
    """
    if request.method == 'POST':
        errors = []
        data = request.get_json()
        try: 
            unorm = current_app.security._username_util.normalize(data["username"])
            user = current_app.security.datastore.find_user(username=unorm) # Find the user
            pnorm = current_app.security._password_util.normalize(data["password"]) # Normalize the password
            if user is None:
                errors.append("Login failed. Please verify your username and password and try again.")
            elif pnorm is None:
                errors.append("The password field cannot be left blank. Please enter your password.")
            else: 
                # Compare the supplied hash with the stored hash for the user
                if verify_password(pnorm, user.password):
                    login_user(user, remember=False) # This will login the user and store a remember me cookie on the clients machine.
                    return jsonify(success=True)
                else: 
                    errors.append("Login failed. Please verify your username and password and try again.")
        except:
            errors.append("An unexpected error occurred.")
        return jsonify({"errors" : errors, "success" : False})
    

 