import React, { useState, useContext, useEffect } from 'react';
import axios from 'axios';
import '../../Css/MultiLevelOptions.css';
import { GlobalContext } from '../../Context/GlobalContext';

const MultiLevelOptions = () => {
  // State to keep track of user selections
  const [category, setCategory] = useState('');
  const [subCategory, setSubCategory] = useState('');
  const [year, setYear] = useState('');
  const [party, setParty] = useState('');
  const [favoriteName, setFavoriteName] = useState('');
  const [favorites, setFavorites] = useState([]);
  const [errorMessages, setErrorMessages] = useState([]);

  // Context to get global state values and functions
  const {serverAddress, serverPort, selectedOptions, setSelectedOptions} = useContext(GlobalContext);

  // Array of years to populate the year dropdown
  const years = ['1973', '1976', '1979', '1982', '1985', '1988', '1991', '1994', '1998', '2002', '2006', '2010', '2014', '2018', '2022'];

  const handleStoredFavorites = async () => {
    try {
      // Send a GET request to the server to get the user's current favorites
      const response = await axios.get(serverAddress + ':' + serverPort + '/api/get_favorites', {
        withCredentials: true, // Include credentials (cookies) in the request
        headers: {
          'Content-Type': 'application/json', 
        },
      });

      setFavorites(response.data.info); // Set the fetched favorites to state

    } catch (error) {
      console.error('Error fetching favorites:', error);
    }
  };

  // Handle category selection (Swedish Election / Swedish Demographic)
  const handleCategoryChange = (e) => {
    console.log("Handle Category Change = " + e.target.value)
    setCategory(e.target.value);
    setSubCategory('');
    setYear('');
    setParty('');
    setFavoriteName('');
    setErrorMessages([]);

    // if the 'favorites' category is selected retrieve the saved favorites from the backend.
    if (e.target.value === 'favorites') { 
      handleStoredFavorites();
    }
  };

  // Handle subcategory selection (Total Votes / Party Votes)
  const handleSubCategoryChange = (e) => {
    setSubCategory(e.target.value);
    setYear('');
    setParty('');
    setFavoriteName(''); // clear the input field
    setErrorMessages([]);
  };

  
  const handleFavoriteChange = (e) => {
    setSubCategory(e.target.value);
    setYear('');
    setParty('');
    setFavoriteName(''); // clear the input field

    // The first element is 'Select Saved Favorite' and is not included
    const index_of_favorite = e.target.options.selectedIndex - 1;
    const selected_favorite_option = favorites[index_of_favorite];
    console.log("Index of fav = " + index_of_favorite);
    console.log(selected_favorite_option);
    setSelectedOptions(selected_favorite_option);
  };


  const saveFavorite = async () => {
    if (favoriteName !== ''){
      // Enrich with the name given by the user
      var enrichedFavoriteOptions = {...selectedOptions}; // Hard copy
      enrichedFavoriteOptions['favoriteName'] = favoriteName;

      // Send the current selected options and its name to backend
      const response = await axios.post(serverAddress + ':' + serverPort + '/api/save_favorite', enrichedFavoriteOptions, {
        withCredentials: true, // Include credentials (cookies) in the request
        headers: {
          'Content-Type': 'application/json', 
        },
      });
    
      if (response.data.success) {
        console.log(favoriteName);
        setErrorMessages([]);
        // After successfully saving, clear the input field
        setFavoriteName('');
      } else {
        console.log("error");
        console.log(favoriteName);
        setErrorMessages(response.data.errors);
        setFavoriteName('');
      }
    }

  };


  const getFavoriteDiv = () => {
    return (
      <div className='favorite-div'>
        <h1>Save as Favorite</h1>
        <input type="text" placeholder="Input Favorite Name"
              value={favoriteName} onChange={(e) => setFavoriteName(e.target.value)}
        />

        {/* Display error messages */}
        {errorMessages.length > 0 && (
          <ul className="error-messages">
              {errorMessages.map((msg, index) => (
                  <li key={index} className="error-message">{msg}</li>
              ))}
          </ul>
        )}

        <button onClick={saveFavorite}>Save Favorite</button>
      </div>
    );
  }

  // Handle year and party selection
  const handleYearChange = (e) => setYear(e.target.value);
  const handlePartyChange = (e) => setParty(e.target.value);

  // This will be called whenever any values in the array 
  // [category, subCategory, year, party] changes
  useEffect(() => {

  console.log("Category = " + category);
  console.log("Sub-Category = " + subCategory);
  console.log("year = "+ year);
  console.log("party = " + party);
  
  setErrorMessages([]); // Remove errors on reload

  // Determine when options have been selected and when a request to the backend should be performed
  if (category === '') {
    setSelectedOptions({'option': ""}); // Reset since no option is selected

  } else if (category === 'Swedish Election Results' && subCategory === '') {
    setSelectedOptions({'option': ""}); // Reset since no option is selected

  } else if (category === 'Swedish Election Results' && subCategory === 'partyVotes' && (!year || !party)) {
    setSelectedOptions({'option': ""}); // Reset since no option is selected

  } else if (category === 'Swedish Demographic') {
    // Route -> /api/swedish_demographic (note that .option is a part of the route)
    setSelectedOptions({'option': "swedish_demographic"}); 

  } else if (category === 'Swedish Election Results' && subCategory === 'totalVotes') {
    // Route -> /api/swedish_election_total_results (note that .option is a part of the route)
    setSelectedOptions({'option': "swedish_election_total_results"}); 

  } else if (category === 'Swedish Election Results' && subCategory === 'allPartyVotes' && year){
    // Route -> /api/swedish_election_party_votes (note that .option is a part of the route)
    setSelectedOptions( 
      {'option': "swedish_election_party_votes",
        'subOption' : "everyParty_one_year",
        'year' : year
      }); 

  } else if (category === 'Swedish Election Results' && subCategory === 'partyVotes' && year && party){
    // Route -> /api/swedish_election_party_votes (note that .option is a part of the route)
    setSelectedOptions({ 
      'option': "swedish_election_party_votes",
      'subOption' : false,
      'year': year,
      'party': party
    });
  }}, [category, subCategory, year, party]);


  // Rendering based on user selections
  return (
    <div className="options-wrapper">
      <h1>Select Options</h1>

      {/* First level menu */}
      <select value={category} onChange={handleCategoryChange}>
        <option value="">-</option>
        <option value="Swedish Election Results">Swedish Election</option>
        <option value="Swedish Demographic">Swedish Demographic</option>
        <option value="favorites">Favorites</option>
      </select>

      {/* Second level menu, visible depending on first level menu selection */}
      {category === 'Swedish Election Results' && (
        <select value={subCategory} onChange={handleSubCategoryChange}>
          <option value="">Select Subcategory</option>
          <option value="totalVotes">Total Votes</option>
          <option value="partyVotes">Single Party Votes</option>
          <option value="allPartyVotes">All Party Votes / Year</option>
        </select>
      )}
      {category === 'favorites' && favorites.length === 0 && ( // No favorites saved
        <select value={subCategory} onChange={handleSubCategoryChange}>
          <option value="noFavSaved">No Favorites Saved</option>
        </select>
      )}
      {category === 'favorites' && favorites.length > 0 && ( // Favorites saved
        <select value={subCategory} onChange={handleFavoriteChange}>
          <option value="">Select Saved Favorite</option>
          {/* Dynamically draw the saved favorites*/}
          {favorites.map((list_element, index) => (
            <option key={index} value={list_element.favoriteName} data_additional={list_element}>
              {list_element.favoriteName}
            </option>
          ))}
        </select>
      )}

      {/* Third level menus: */}
      {subCategory === 'allPartyVotes' && (
        <div>
           <select value={year} onChange={handleYearChange}>
                <option value="">Select Election Year</option>
                {/* Create an 'Option' element for each year */}
                {years.map((year) => (
                    <option key={year} value={year}>{year}</option>
                ))}
          </select>
        </div>
      )}

      {/* Third level menus: */}
      {subCategory === 'partyVotes' && (
        <div>
            <select value={party} onChange={handlePartyChange}>
                <option value="">Select Party</option>
                <option value="Moderaterna">Moderaterna (M)</option>
                <option value="Centerpartiet">Centerpartiet (C)</option>
                <option value="Liberalerna">Liberalerna (FP)</option>
                <option value="Kristdemokraterna">Kristdemokraterna (KD)</option>
                <option value="Socialdemokraterna">Socialdemokraterna (S)</option>
                <option value="Sverigedemokraterna">Sverigedemokraterna (SD)</option>
                <option value="Överiga partier">Other</option>
                <option value="ogiltiga valsedlar">Invalid</option>
                <option value="ej röstande">Non-Voters</option>
           </select>

           <select value={year} onChange={handleYearChange}>
                <option value="">Select Election Year</option>
                {/* Create an 'Option' element for each year */}
                <option key='All' value='All'>All</option>
                {years.map((year) => (
                    <option key={year} value={year}>{year}</option>
                ))}
          </select>
        </div>
      )}

      {/* Final display based on selection */}
      <div>
            {category === 'Swedish Demographic' && subCategory === '' && (
              <>
                <p>Selected: {category}</p>
                {getFavoriteDiv()}
              </>
            )}

            {category === 'Swedish Election Results' && subCategory === 'totalVotes' && (
              <>
                <p>Showing Total Votes for {category}</p>
                {getFavoriteDiv()}
              </>
            )}

            {category === 'Swedish Election Results' && subCategory === 'partyVotes' && year && party && (
              <>
                <p>Showing Votes for {party} in {year} Election</p>
                {getFavoriteDiv()}
              </>
            )}

            {category === 'Swedish Election Results' && subCategory === 'allPartyVotes' && year && (
              <>
                <p>Showing All Party Votes for the Year {year}</p>
                {getFavoriteDiv()}
              </>
            )}

      </div>
    </div>
  );
};

export default MultiLevelOptions;