import React, { useEffect, useRef, useContext } from 'react';
import * as d3 from 'd3';
import axios from 'axios';
import { GlobalContext } from '../../Context/GlobalContext';

const BarChart = ({ selectedKey }) => {
  const {serverAddress, serverPort, selectedOptions} = useContext(GlobalContext);
  let svgRef = useRef(); // Create a ref to access the SVG element across renders


  useEffect( () => {
     const svg = d3.select(svgRef.current);  // Select the current SVG element referenced by svgRef  
      let response = {};
      
      // Fetch data from the API based on the selected options
      const fetchData = async (e) => {
        if (selectedKey > 0 && selectedOptions.option != ''){
          try {
            // Make a GET request to the API with the selected option and key
            response = await axios.get(serverAddress + ':' + serverPort + '/api/' + selectedOptions.option +'?code='+selectedKey, {
            withCredentials: true, // Include credentials (cookies) in the request
            headers: {
              'Content-Type': 'application/json', 
            },
          });
          } catch (error) {
            console.log(e);
          }
        } else {
          // Clear previous SVG content if no option or county is selected
          svg.selectAll('*').remove();
        } 

        if (Object.keys(response).length > 0 && response.data.success){
              /* 
              * The variable `selectedOptions.option` determines what data is drawn on the chart. 
              * Data should be handled separately for each type of graph based on the `selectedOptions.option` value.
              * 
              * SelectedOptions.option is one of :
            *                               swedish_election_allPartyVotes,
              *                             swedish_election_total_results,
              *                             swedish_election_party_votes,
              *                             swedish_demographic,
              */

              // Set up common dimensions and margins
              const margin = { top: 20, right: 60, bottom: 30, left: 60 };
              const width = 1000 - margin.left - margin.right;
              const height = 500 - margin.top - margin.bottom;

              // Clear previous SVG content
              svg.selectAll('*').remove();


              if (selectedOptions.option === 'swedish_election_total_results'){
                /* 
                * If this option is selected show a number instead of a graph
                */ 
                const total_votes_in_county = response.data.info.query_data;
                const formattedNumber = d3.format(",")(total_votes_in_county);

                svg.append("text")
                  .attr("x", width / 2)
                  .attr("y", height / 2)
                  .attr("text-anchor", "middle")
                  .attr("white-space", "normal")
                  .attr("font-size", "70px")
                  .attr("fill", "#3d96b1")
                  .attr("stroke-width", "2px")
                  .attr("stroke", "#3d96b1")
                  .attr("font-family", "source-code-pro, Menlo, Monaco, Consolas, 'Courier New', monospace")
                  .text(formattedNumber);
              }
              

              if (selectedOptions.option === 'swedish_election_party_votes' && selectedOptions.subOption === 'everyParty_one_year'){
                const partiesKeys = Object.keys(response.data.info.query_data);
                
                const partyAbbreviations = {
                  "Centerpartiet": "C",
                  "Kristdemokraterna": "KD",
                  "Liberalerna": "L",
                  "Miljöpartiet": "MP",
                  "Moderaterna": "M",
                  "Socialdemokraterna": "S",
                  "Sverigedemokraterna": "SD",
                  "Vänsterpartiet": "V",
                  "Överiga partier": "Other",
                  "ogiltiga valsedlar": "Invalid",
                  "ej röstande": "No Vote"
                };

                const tempData = {}; // Votes for each party in a specific year 
                for (const partyName of partiesKeys){
                  const votes = response.data.info.query_data[partyName].votes_per_year[selectedOptions.year];
                  const partyAbbreviation = partyAbbreviations[partyName] || partyName; // Attempt to translate into abbreviations to save space
                  tempData[partyAbbreviation] = votes;
                }

                const partyKeys = Object.keys(tempData);
                const values = Object.values(tempData);

                const d3PreparedData = partyKeys.map((partyAbbrev, index) => ({
                  'party': partyAbbrev,   // Keep year as a number for numerical operations
                  'value': values[index]  // Get the corresponding value
                }));


                // Customize the colors as desired
                  /* Centerpartiet = #009933, 
                  * Kristdemokraterna = #000077
                  * Liberalerna = #006AB3
                  * Miljöpartiet = #83CF39
                  * Moderaterna = #52BDEC
                  * Socialdemokraterna = #E8112d
                  * Sverigedemokraterna = #DDDD00
                  * Vänsterpartiet = #DA291C
                  * Överiga partier = #9E9E9E 
                  * ogiltiga valsedlar = #8B5E3C 
                  * ej röstande = #D3D3D3 
                  */
                const colorScale = d3.scaleOrdinal()
                  .domain(Object.values(partyAbbreviations))
                  .range(["#009933", "#000077", "#006AB3", "#83CF39", "#52BDEC", "#E8112d", "#DDDD00", "#DA291C", "#9E9E9E", "#8B5E3C", "#D3D3D3"]);             

                // Create the graph:
                // Set up scales
                const xScale = d3.scaleBand()
                  .domain(partyKeys) 
                  .range([0, width])
                  .padding(0.1);
            
                const yScale = d3.scaleLinear()
                  .domain([0, d3.max(values)])  // Highest value in dataset
                  .nice()
                  .range([height, 0]);

                // Create x-axis
                svg.append('g')
                  .attr('transform', `translate(${margin.left}, ${height + margin.top})`)
                  .call(d3.axisBottom(xScale))
                  .selectAll('path, line, text')  // Select the axis lines and text
                  .attr('stroke', 'white')        // Set the color of the lines to white
                  .attr('stroke-width', 0)        // Set the width of the text
                  .attr('fill', 'white')          // Set the color of the text to white
                  .attr('font-size', '18px');

                // Create y-axis
                svg.append('g')
                  .attr('transform', `translate(${margin.left}, ${margin.top})`)
                  .call(d3.axisLeft(yScale))
                  .selectAll('path, line, text')  // Select the axis lines and text
                  .attr('stroke', 'white')        // Set the color of the lines to white
                  .attr('stroke-width', 0)        // Set the width of the text
                  .attr('fill', 'white')          // Set the color of the text to white
                  .attr('font-size', '14px');

                // Create bars
                svg.selectAll('.bar')
                  .data(d3PreparedData)
                  .enter().append('rect')
                  .attr('class', 'bar')
                  .attr('x', d => xScale(d.party) + margin.left)
                  .attr('y', d => yScale(d.value) + margin.top)
                  .attr('width', xScale.bandwidth())
                  .attr('height', d => height - yScale(d.value))
                  .attr('fill', d => colorScale(d.party));
              }


            if (selectedOptions.option === 'swedish_election_party_votes' && selectedOptions.subOption === false){
                /* 
                * If this option is selected create a graph or show a number depending on the options.  
                * Show votes for a single party over a time period
                */ 
                
                let d3PreparedData = {};
                const yearKeys = Object.keys(response.data.info.query_data[selectedOptions.party].votes_per_year);
                const values = Object.values(response.data.info.query_data[selectedOptions.party].votes_per_year);

                if (selectedOptions.year === 'All'){
                  d3PreparedData = yearKeys.map((year, index) => ({
                    'year': year,   // Keep year as a number for numerical operations
                    'value': values[index]  // Get the corresponding value
                  }));
                } else {

                  const total_votes_in_county_for_party = response.data.info.query_data[selectedOptions.party].votes_per_year[selectedOptions.year];
                  const formattedNumber = d3.format(",")(total_votes_in_county_for_party);

                  svg.append("text")
                    .attr("x", width / 2)
                    .attr("y", height / 2)
                    .attr("text-anchor", "middle")
                    .attr("white-space", "normal")
                    .attr("font-size", "70px")
                    .attr("fill", "#3d96b1")
                    .attr("stroke-width", "2px")
                    .attr("stroke", "#3d96b1")
                    .attr("font-family", "source-code-pro, Menlo, Monaco, Consolas, 'Courier New', monospace")
                    .text(formattedNumber);
                  return; //Exit, don't create a graph
                }

                // Create the graph:
                // Set up scales
                const xScale = d3.scaleBand()
                  .domain(yearKeys)  
                  .range([0, width])
                  .padding(0.1);
            
                const yScale = d3.scaleLinear()
                  .domain([0, d3.max(values)])  // Highest value in dataset
                  .nice()
                  .range([height, 0]);

                // Create x-axis
                svg.append('g')
                  .attr('transform', `translate(${margin.left}, ${height + margin.top})`)
                  .call(d3.axisBottom(xScale))
                  .selectAll('path, line, text')  // Select the axis lines and text
                  .attr('stroke', 'white')        // Set the color of the lines to white
                  .attr('stroke-width', 0)        // Set the width of the text
                  .attr('fill', 'white')          // Set the color of the text to white
                  .attr('font-size', '16px');

                // Create y-axis
                svg.append('g')
                  .attr('transform', `translate(${margin.left}, ${margin.top})`)
                  .call(d3.axisLeft(yScale))
                  .selectAll('path, line, text')  // Select the axis lines and text
                  .attr('stroke', 'white')        // Set the color of the lines to white
                  .attr('stroke-width', 0)        // Set the width of the text
                  .attr('fill', 'white')          // Set the color of the text to white
                  .attr('font-size', '14px');

                // Create bars
                svg.selectAll('.bar')
                  .data(d3PreparedData)
                  .enter().append('rect')
                  .attr('class', 'bar')
                  .attr('x', d => xScale(d.year) + margin.left)
                  .attr('y', d => yScale(d.value) + margin.top)
                  .attr('width', xScale.bandwidth())
                  .attr('height', d => height - yScale(d.value))
                  .attr('fill', '#3D96B1');
                }


              if (selectedOptions.option === 'swedish_demographic'){
                    /* 
                    * If this option is selected create a graph
                    */ 
                    const yearKeys = Object.keys(response.data.info.query_data).map(year => parseFloat(year));
                    const values = Object.values(response.data.info.query_data);

                    const d3PreparedData = yearKeys.map((year, index) => ({
                      'year': year,   // Keep year as a number for numerical operations
                      'value': values[index]  // Get the corresponding value
                    }));

                    // Create the graph:
                    // Set up scales
                    const xScale = d3.scaleBand()
                      .domain(yearKeys)  
                      .range([0, width])
                      .padding(0.1);
                  
                    const yScale = d3.scaleLinear()
                      .domain([0, d3.max(values)])  // Highest value in dataset
                      .nice()
                      .range([height, 0]);

                    // Create x-axis
                    svg.append('g')
                      .attr('transform', `translate(${margin.left}, ${height + margin.top})`)
                      .call(d3.axisBottom(xScale))
                      .selectAll('path, line, text')  // Select the axis lines and text
                      .attr('stroke', 'white')        // Set the color of the lines to white
                      .attr('stroke-width', 0)        // Set the width of the text
                      .attr('fill', 'white')          // Set the color of the text to white
                      .attr('font-size', '12px');

                    // Create y-axis
                    svg.append('g')
                      .attr('transform', `translate(${margin.left}, ${margin.top})`)
                      .call(d3.axisLeft(yScale))
                      .selectAll('path, line, text')  // Select the axis lines and text
                      .attr('stroke', 'white')        // Set the color of the lines to white
                      .attr('stroke-width', 0)        // Set the width of the text
                      .attr('fill', 'white')          // Set the color of the text to white
                      .attr('font-size', '20px');

                    // Create bars
                    svg.selectAll('.bar')
                      .data(d3PreparedData)
                      .enter().append('rect')
                      .attr('class', 'bar')
                      .attr('x', d => xScale(d.year) + margin.left)
                      .attr('y', d => yScale(d.value) + margin.top)
                      .attr('width', xScale.bandwidth())
                      .attr('height', d => height - yScale(d.value))
                      .attr('fill', '#3D96B1');
              }
          }
    };

    fetchData();

  }, [selectedKey, selectedOptions]); // Whenever these are changed invoke useEffect()

  return (
    <svg ref={svgRef} width={1000} height={500}></svg>
  );
};

export default BarChart;
