import React, { useState, useContext } from 'react';
import axios from 'axios';
import '../../Css/SignForm.css';
//React-icons imports
import { MdEmail } from "react-icons/md";
import { FaUser } from "react-icons/fa";
import { FaLock } from "react-icons/fa";

import { useNavigate } from 'react-router-dom'; // Import useNavigate hook
import { GlobalContext } from '../../Context/GlobalContext';

export const SignUpForm = ({ onLoginClick }) => {
    const [username, setUsername] = useState('');
    const [email, setEmail] = useState('');
    const [password, setPassword] = useState('');
    const [passwordRepeat, setPasswordRepeat] = useState('');
    const [errorMessages, setErrorMessages] = useState([]);
    const {serverAddress, serverPort, setLoggedInStatus} = useContext(GlobalContext);
    const navigate = useNavigate();

    // Call the onLoginClick function passed as prop
    const handleLoginClick = () => {
        onLoginClick();
      };

    // State reactive handler functions:
    const handleUsernameChange = (e) => {
      setUsername(e.target.value);
    };

    const handleEmailChange = (e) => {
        setEmail(e.target.value);
    };

    const handlePasswordChange = (e) => {
      setPassword(e.target.value);
    };

    const handlePasswordRepeatChange = (e) => {
        setPasswordRepeat(e.target.value);
    };

     // Function to validate the form inputs
    const validateForm = () => {
      const errors = [];

      // Validate username length
      if (username.length < 4 || username.length > 20){
        errors.push("Username must be between 4 and 20 characters.");
      }

      // Check if passwords match
      if (password !== passwordRepeat){
        errors.push("Passwords do not match");
      }

      // Validate password length
      if (password.length < 5) {
        errors.push("Password must be at least 5 characters long.");
      } 
      if (password.length > 255) {
        errors.push("Password must be at shorter than 255 characters long.");
      }

      // Validate email format using a regex pattern
      const emailRegex = /^\S+@+\S+$/;
      if (emailRegex.test(email.toLowerCase()) === false){
        errors.push("Please enter a valid email address.");
      }

      return errors;
    }

    // Handler for when the form is submitted
    const handleRegister = async (e) => {
      e.preventDefault(); // Prevent default form submission
      setErrorMessages([])
      // Validate the form before sending request to the backend.
      const validationErrors = validateForm();
      if (validationErrors.length > 0){
        setErrorMessages(validationErrors); // Display validation errors
        return; //Stop registration process if there are errors
      }

      // Prepare registration data
      const registrationData = {
        email,
        username,
        password
      };

      try {
        // Send a POST request to the server to register the user
        const response = await axios.post(serverAddress + ':' + serverPort + '/api/register', registrationData, {
          headers: {
              'Content-Type': 'application/json', 
          },
        });

        if (response.data.success) {
          setErrorMessages([]); // Clear any error messages
          setLoggedInStatus(true); // Set the logged-in status in global context
          navigate('/dashboard'); // Redirect user to the dashboard page when successful

        } else {
          setErrorMessages(response.data.errors); // Display errors returned from the server
        }

      } catch (error) {
        setErrorMessages(['An unexpected error occurred.']); 
      }
    };

  return (
    <div className='wrapper'>
      <form onSubmit={handleRegister}>    
        <h1>Register</h1>

        {/* Display error messages */}
        {errorMessages.length > 0 && (
                    <ul className="error-messages">
                        {errorMessages.map((msg, index) => (
                            <li key={index} className="error-message">{msg}</li>
                        ))}
                    </ul>
        )}

        {/* Email input field */}
        <div className="input-box">
            <input type="email" placeholder='Email' value={email} onChange={handleEmailChange} required/>
            <MdEmail className='icon'/>
        </div>

        {/* Username input field */}
        <div className="input-box">
            <input type="text" placeholder='Username' value={username} onChange={handleUsernameChange} required/>
            <FaUser className='icon'/>
        </div>

        {/* Password input field */}
        <div className="input-box">
            <input type="password" placeholder='Password' value={password} onChange={handlePasswordChange} required/>
            <FaLock className='icon'/>
        </div>

        {/* Repeat password input field */}
        <div className="input-box">
            <input type="password" placeholder='Repeat Password' value={passwordRepeat} onChange={handlePasswordRepeatChange} required/>
            <FaLock className='icon'/>
        </div>

        {/* Link to switch to login form */}
        <div className="register-link">
                <p>Already have an account? <a href="/" onClick={handleLoginClick}>Log in</a></p>
              
            </div>

        {/* Submit button for registration */}
        <button style={{marginTop: 20  }} type='submit'>Register</button>
    
      </form>
    </div>
  )
}
